#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <ncurses.h>
#include <unistd.h>
#include <termios.h>
#include <limits.h>
#include <time.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>

#include "hash.h"
#include "structure.h"
#include "base.h"
#include "move.h"

// ---------------------------------------------------------------------------------------------------- //
double sensitivity = 0.3 ;
double fov = 90.0 ;
double speed = 1.0 ;
// ---------------------------------------------------------------------------------------------------- //

double camx ;
double camy ;
double camz ;

double rot_hz ;
double rot_vt ;

double tan_fov ;

bool has_changed ;

void init_csts() {
    camx = 0.0 ;
    camy = 0.0 ;
    camz = 0.0 ;
    rot_hz = 0.0 ;
    rot_vt = 0.0 ;
    tan_fov = tan((fov * 3.14159 / 180.0) / 2.0) ;
}

void playerActions() {
    SDL_Event event;
    while(SDL_PollEvent(&event)) {
        switch (event.type) {
            case SDL_QUIT:
                break;
            case SDL_MOUSEMOTION:
                has_changed = true ;
                rot_hz -= sensitivity * event.motion.xrel / 100.0 ;
                rot_vt -= sensitivity * event.motion.yrel / 100.0 ;
                if(rot_hz >= 2*3.141592) {
                    rot_hz -= 2*3.141592 ;
                } else if(rot_hz < 0.0) {
                    rot_hz += 2*3.141592 ;
                }
                if(rot_vt >= 2*3.141592) {
                    rot_vt -= 2*3.141592 ;
                } else if(rot_vt < 0.0) {
                    rot_vt += 2*3.141592 ;
                }
                break;
            case SDL_KEYDOWN:
                has_changed = true ;
                switch (event.key.keysym.sym) {
                    case SDLK_z:
                        camz += speed*cos(rot_hz)*cos(rot_vt);
                        camx -= speed*sin(rot_hz)*cos(rot_vt);
                        camy += speed*sin(rot_vt);
                        break;
                    case SDLK_q:
                        camx -= speed*cos(rot_hz);
                        camz -= speed*sin(rot_hz);
                        break;
                    case SDLK_s:
                        camz -= speed*cos(rot_hz)*cos(rot_vt);
                        camx += speed*sin(rot_hz)*cos(rot_vt);
                        camy -= speed*sin(rot_vt);
                        break;
                    case SDLK_d:
                        camx += speed*cos(rot_hz);
                        camz += speed*sin(rot_hz);
                        break;

                    case SDLK_t:
                        fprintf(stderr, "Killed.\n") ;
                        exit(1) ;
                        break;
                    
                    case SDLK_a:
                        rot_hz -= sensitivity ;
                        break;
                    case SDLK_e:
                        rot_hz += sensitivity ;
                        break;
                    
                    case SDLK_p:
                        rot_vt -= sensitivity ;
                        break;
                    case SDLK_m:
                        rot_vt += sensitivity ;
                        break;
                    
                    default:
                        break;
                }
        }
    }
}